"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFilesFromTemplate = exports.generateDataBind = exports.isSlot = exports.pullActionToListByInstances = exports.getOriginComponentAndActionList = exports.runGenerateCore = void 0;
const weapps_core_1 = require("../../weapps-core");
const common_1 = require("../types/common");
const path_1 = __importDefault(require("path"));
const template_1 = __importDefault(require("../template"));
const util_1 = require("../util");
const config_1 = require("../config");
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const style_1 = require("../util/style");
const types_1 = require("../../types");
const dataSource_1 = require("../../utils/dataSource");
const cals_1 = require("@cloudbase/cals");
const lodash_1 = require("lodash");
const _1 = require(".");
const common_2 = require("../util/common");
async function runGenerateCore(props) {
    const { appBuildDir, cals, appData, subAppDataList = [], dependencies = [], buildTypeList, appKey, basename, extraData = {
        isComposite: false,
        compProps: {},
    }, i18nConfig, fileCodeMap, isSandbox = false, deployOptions, _indexPage, processCssUnit, rpxCalcRoot, isPrivateMode = false, expirationStartTimesnap, } = props;
    const { domain } = appData.extra;
    const allAppDataList = [appData].concat(subAppDataList);
    // 安装插件依赖
    const deps = {};
    allAppDataList.map((app) => {
        Object.entries(app.npmDependencies).forEach(([name, version]) => {
            deps[name] = version;
        });
    });
    // 合并组件库的公共npm
    dependencies.map((compLb) => {
        if (compLb.isComposite && compLb.compLibCommonResource) {
            Object.assign(deps, compLb.compLibCommonResource.npm);
        }
    });
    /**
     * 组模块目录结构待定
     * 不生成子包
     */
    const usedComps = (0, cals_1.getUsedComps)([cals], dependencies, {
        streamline: true,
    });
    // await Promise.all(
    //   allAppDataList.map(async (appData) => {
    const { rootPath = '' } = appData;
    console.log(`Generating files for ${rootPath ? `Sub app ${rootPath}` : 'Main app'}`);
    const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `${rootPath}` : '');
    await generateCodeFromTpl(appKey, appData, dstDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployOptions, domain, dependencies, processCssUnit, isPrivateMode, usedComps);
    await writeLowCodeFiles(appData, dstDir, fileCodeMap, rootPath, processCssUnit, rpxCalcRoot);
    //   }),
    // );
    await generateAppData(cals, dependencies, appBuildDir, deployOptions, usedComps, fileCodeMap);
    await generateRouterFile(allAppDataList, appBuildDir, basename, buildTypeList, fileCodeMap, isSandbox, _indexPage, expirationStartTimesnap);
}
exports.runGenerateCore = runGenerateCore;
/**
 * 根据复合组件库获取到所有包含抽象节点的数据
 * @param dependencies
 * @returns
 */
function getGenericCompFromDep(dependencies = []) {
    const genericCompMap = {};
    dependencies.forEach((compLib) => {
        if (compLib.isComposite) {
            compLib.components.forEach((component) => {
                (0, util_1.deepDealComponentSchemaJson)(Object.values(component.componentInstances)[0], (componentSchema) => {
                    if (componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.genericComp) {
                        genericCompMap[`${component.materialName}:${component.name}`] = componentSchema.genericComp;
                    }
                });
            });
        }
    });
    return genericCompMap;
}
function getOriginComponentAndActionList(componentSchema, dependencies, originComponentList = [], originActionList = [], genericCompMap = {}) {
    const fieldSchema = new react_schema_renderer_1.Schema(componentSchema);
    if (fieldSchema.isObject()) {
        const { 'x-props': xProps } = fieldSchema;
        if (xProps) {
            const { listenerInstances, sourceKey } = xProps;
            // 属于抽象节点属性
            if (genericCompMap[sourceKey]) {
                const genericProps = genericCompMap[sourceKey];
                // @ts-ignore
                const genericComp = xProps.data[genericProps.propName];
                pullComponentToListByInstance(genericComp, originComponentList, dependencies);
            }
            pullComponentToListByInstance(sourceKey, originComponentList, dependencies);
            pullActionToListByInstances(listenerInstances, originActionList, dependencies);
        }
        const filedSchemaProperties = fieldSchema.properties || {};
        Object.keys(filedSchemaProperties).forEach((key) => {
            const schema = filedSchemaProperties[key];
            const schemaJson = schema;
            getOriginComponentAndActionList(schemaJson, dependencies, originComponentList, originActionList, genericCompMap);
        });
    }
    return {
        originComponentList,
        originActionList,
    };
}
exports.getOriginComponentAndActionList = getOriginComponentAndActionList;
function pullActionToListByInstances(listenerInstances, originActionList, fixedDependencies) {
    if (!listenerInstances || !listenerInstances.length) {
        return;
    }
    listenerInstances.map((pageListenerInstance) => {
        const { sourceKey, type } = pageListenerInstance;
        const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
        const material = fixedDependencies.find((m) => m.name === materialName);
        const actionKey = `${materialName}_${name}`;
        const isExistAction = originActionList.find((item) => item.key === actionKey);
        if (!isExistAction) {
            originActionList.push({
                name,
                materialName,
                materialVersion: material === null || material === void 0 ? void 0 : material.version,
                key: actionKey,
                type,
                variableName,
                entries: material === null || material === void 0 ? void 0 : material.entries,
            });
        }
    });
}
exports.pullActionToListByInstances = pullActionToListByInstances;
function pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies) {
    var _a;
    const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
    const componentKey = `${materialName}_${name}`;
    const isExistComponent = originComponentList.find((item) => item.key === componentKey);
    if (!isExistComponent) {
        const foundOne = fixedDependencies.find((m) => m.name === materialName);
        if (!foundOne)
            return;
        const { schemaVersion = '' } = foundOne;
        let isPlainProps = false;
        try {
            if (Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0]) >= 3) {
                isPlainProps = true;
            }
        }
        catch (e) { }
        originComponentList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: foundOne.version,
            isComposite: foundOne.isComposite,
            key: componentKey,
            variableName: variableName || '',
            isPlainProps,
            entries: foundOne === null || foundOne === void 0 ? void 0 : foundOne.entries,
        });
    }
}
function isSlot(comp) {
    return comp.path && !comp['x-props'];
}
exports.isSlot = isSlot;
function generateDataBind(bind) {
    let expr = '';
    const { type, value } = bind;
    if (type === weapps_core_1.PropBindType.forItem) {
        expr = `forItems.${value}`;
    }
    else if (type === weapps_core_1.PropBindType.expression) {
        expr = value;
    }
    else if (type === weapps_core_1.PropBindType.prop) {
        const isNegated = value.startsWith('!');
        const originValue = value.replace(/^!/, '');
        expr = `${isNegated ? '!' : ''}$comp.props.data.${originValue}`;
    }
    else {
        const isGlobal = value.startsWith('global.');
        const isCopositive = value.startsWith('comp-') || value.startsWith('$comp_');
        const PREFIX_MAP = {
            [weapps_core_1.PropBindType.state]: 'state',
            [weapps_core_1.PropBindType.computed]: 'computed',
            [weapps_core_1.PropBindType.dataVar]: 'dataVar',
            [weapps_core_1.PropBindType.stateData]: 'dataset.state',
            [weapps_core_1.PropBindType.paramData]: 'dataset.params',
        };
        switch (type) {
            case weapps_core_1.PropBindType.state:
            case weapps_core_1.PropBindType.computed:
            case weapps_core_1.PropBindType.dataVar:
            case weapps_core_1.PropBindType.stateData:
            case weapps_core_1.PropBindType.paramData: {
                if (isCopositive) {
                    expr = value
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
                else {
                    expr = value
                        .replace(/^\w+./, `${isGlobal ? 'app' : '$page'}.${PREFIX_MAP[type]}.`)
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
            }
        }
    }
    return expr;
}
exports.generateDataBind = generateDataBind;
async function generateRouterFile(allAppDataList, appBuildDir, basename = '', buildTypeList = [], fileCodeMap, isSandbox, _indexPage = false, expirationStartTimesnap = 0) {
    const routerImports = [];
    const routerRenders = [];
    await Promise.all(allAppDataList.map(async (data) => {
        const { pageInstanceList, rootPath = '' } = data;
        const pageFilePath = rootPath ? `/src/packages/${rootPath}/` : '/src/';
        // 判断app环境才进行加载引入
        pageInstanceList.map((pageInstance) => {
            const pageId = [rootPath, pageInstance.id].filter((i) => !!i).join('_');
            const pagePath = rootPath ? `/${rootPath}/${pageInstance.id}` : `/${pageInstance.id}`;
            if (isSandbox) {
                routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
            }
            else {
                routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
                // routerImports.push(
                //   `const Page${pageId} = React.lazy(() => import('${pageFilePath}pages/${pageInstance.id}/index'));`
                // );
            }
            routerRenders.push({
                id: pageId,
                path: pagePath,
                isHome: !rootPath && !!pageInstance.isHome,
                root: rootPath || '',
                // Route 这里需要加 () => <Pageexample /> 才能触发热更新，具体原因还没查
                component: `${config_1.REPLACE_SIGN}() => <Page${pageId} />${config_1.REPLACE_SIGN}`,
            });
        });
    }));
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/index.tpl.jsx': {
                expirationStartTimesnap,
                routerImports: routerImports.join('\n'),
                routerRenders: (0, util_1.JsonToStringWithVariableName)(routerRenders),
                basename,
                isHash: isSandbox,
                context: {
                    history: { _indexPage },
                    basic: (0, common_1.buildAsXPageByBuildType)(buildTypeList),
                    checkPageAuth: (0, common_1.buildAsXPageByBuildType)(buildTypeList) || (0, common_1.buildAsAdminPortalByBuildType)(buildTypeList)
                        ? 'function(){ return true }'
                        : 'undefined',
                },
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
async function generateAppData(cals, dependencies, appBuildDir, deployOptions, usedComps, fileCodeMap) {
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/data.tpl.js': {
                /**
                 * 不添加缩进以精简大小
                 */
                cals: JSON.stringify({
                    ...cals,
                    version: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.version) || '',
                    resources: undefined,
                    items: (cals.items || []).map((page) => {
                        return {
                            ...page,
                            resources: undefined,
                        };
                    }),
                    // 数据源配置单独生成
                    dataSources: [],
                    extra: {
                        ...(cals.extra || {}),
                        // 过滤设计态属性
                        pageTree: undefined,
                        compHiddenConfig: undefined,
                        componentPropsDatas: undefined,
                        processPages: undefined,
                        lastDependencies: undefined,
                        conditionalRendering: undefined,
                        dataForm: undefined,
                        compConfig: undefined,
                    },
                }),
                /**
                 * 不添加缩进以精简大小
                 */
                dependencies: JSON.stringify(dependencies
                    .filter((lib) => lib.name === config_1.OFFICIAL_LIB_KEY || !!usedComps[lib.name])
                    .map((lib) => {
                    return {
                        ...lib,
                        components: [],
                        actions: undefined,
                        srcZipUrl: undefined,
                        mpPkgUrl: undefined,
                        mainJsPath: undefined,
                        mainCssPath: undefined,
                    };
                })),
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
async function writeLowCodeFiles(appData, appBuildDir, fileCodeMap, rootPath, processCssUnit, rpxCalcRoot) {
    var _a;
    const lowcodeRootDir = path_1.default.join(appBuildDir, 'lowcode');
    const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: 'global' }, appData.codeModules, 'app');
    const themeCode = codeModules.find((mod) => mod.type === cals_1.ECodeType.THEME);
    await Promise.all(codeModules.map((m) => writeCode2file(m, 'global', undefined, processCssUnit, rpxCalcRoot)).concat());
    const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
    await writeCode2file({
        type: 'index',
        name: 'entry',
        path: 'global/index',
        code: `${['state', 'computed', 'lifecycle']
            .map((key) => `import ${key} from './${key}'`)
            .concat("import common from '../app/common'")
            .concat(appData.pageInstanceList.map((page) => `import * as ${(0, common_2.normalizePageId)(page.id)} from './${page.id}/index'`))
            .concat('')
            .join(';\n')}
export default {
  global: {
    state,
    computed,
    lifecycle,
    style: [{code:\`${styleModule
            ? ((_a = fileCodeMap[path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)('global', styleModule, {
                style: '.css',
            }))]) === null || _a === void 0 ? void 0 : _a.code) || ''
            : ''}\`}],
    common
  },
  ${appData.pageInstanceList.map((page) => `['${page.id}']: ${(0, common_2.normalizePageId)(page.id)}`).join(',\n')}
}
`,
    }, 'global', processCssUnit, rpxCalcRoot);
    await Promise.all(appData.pageInstanceList.map(async (page) => {
        var _a;
        const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: page.id }, page.codeModules, 'page');
        await Promise.all(codeModules.map((m) => writeCode2file(m, page.id, page, processCssUnit, rpxCalcRoot)));
        const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
        await writeCode2file({
            type: 'index',
            name: 'entry',
            path: `${page.id}/index`,
            code: `export { default as state } from './state';
        export { default as  computed } from './computed';
        export { default as lifecycle } from './lifecycle';
        // export style from './style';
        export const style = [{code: \`${styleModule
                ? ((_a = fileCodeMap[path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)(page.id, styleModule, {
                    style: '.css',
                }))]) === null || _a === void 0 ? void 0 : _a.code) || ''
                : ''}\`}];
        export { ${(0, common_2.normalizePageId)(page.id)} as handler} from '../../app/handlers';
        `,
        }, 'global', processCssUnit, rpxCalcRoot);
    }));
    async function writeCode2file(mod, pageId, page, processCssUnit = 'rem', rpxCalcRoot) {
        const file = path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)(pageId, mod, {
            style: '.css',
        }));
        let code = mod.code || '';
        let weappsApiPrefix = '';
        if (mod.type !== cals_1.ECodeType.THEME && mod.type !== cals_1.ECodeType.STYLE && mod.type !== 'index') {
            const isIncludeAppDot = !!code
                // 先去掉模板代码
                .replace('通过 app.common.[name].xxx 访问这里定义的方法或值', '')
                // 再匹配
                .match(/[^'"\w]?app[^'"\w]/g);
            if (isIncludeAppDot) {
                // weappsApiPrefix = `import { app } from '${path
                //   .relative(path.dirname(file), `${appBuildDir}/app/global-api`)
                //   .replace(/\\/g, '/')}';\nconst $app = app;\n`;
                // weappsApiPrefix = `const app = window.app;\nconst $app = app;\n`;
                weappsApiPrefix = `${
                /* !isSandbox */ false
                    ? `import { app } from '${path_1.default
                        .relative(path_1.default.dirname(file), `${appBuildDir}/app/global-api`)
                        .replace(/\\/g, '/')}';`
                    : config_1.LOWCODE_INJECT_APP_TEMPLATE}\nconst $app = new Proxy({}, { get: function(obj, prop){ return app[prop] }});`;
            }
            if (page) {
                // weappsApiPrefix = `import { $page } from '${(path.dirname(file), appBuildDir).replace(
                //   /\\/g,
                //   '/',
                // )}/pages/${pageId}/api';`;
                // weappsApiPrefix = `cosnt $page = window.$page`;
            }
        }
        if (mod.type === 'style') {
            // 生成页面样式
            let pageStyleString = '';
            if (page) {
                const pageClass = [rootPath, page.id].filter(Boolean).join('-');
                pageStyleString = (0, weapps_core_1.toCssText)((0, cals_1.processCommonStyle2CSSProperties)(page.style, {
                    defaultUnit: processCssUnit,
                    rpxCalcRoot,
                }), `.weapps-page-${pageClass}`);
            }
            code = await (0, style_1.processLess)(((themeCode === null || themeCode === void 0 ? void 0 : themeCode.code) || '') + code, processCssUnit);
        }
        if (mod.type === 'theme') {
            return;
        }
        fileCodeMap[file] = {
            code: `${weappsApiPrefix}\n${code}`,
        };
    }
}
async function generateCodeFromTpl(
/**
 * 应用ID
 */
appKey, 
/**
 * 应用数据
 */
appData, appBuildDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployOptions, 
/**
 * 静态域名
 */
domain, dependencies, processCssUnit, isPrivateMode, usedCmps) {
    var _a, _b;
    const pageIds = [];
    const pageModules = {};
    appData.pageInstanceList.map((p) => {
        pageIds.push(p.id);
        pageModules[p.id] = { pageNameVar: (0, common_2.normalizePageId)(p.id), codes: p.codeModules };
    });
    const yyptConfig = await (0, util_1.getYyptConfigInfo)(extraData);
    const { staticDatasourceProfile = true } = (0, cals_1.parseVersion)('', dependencies);
    const genericCompMap = getGenericCompFromDep(dependencies);
    // originComponentList 包含了引入的组件与抽象节点绑定的组件
    const { originComponentList, originActionList } = appData.pageInstanceList.reduce((res, pageInstance) => {
        const { componentSchemaJson, listenerInstances: pageListenerInstances } = pageInstance;
        const { originComponentList = [], originActionList = [] } = getOriginComponentAndActionList(componentSchemaJson, dependencies, [], [], genericCompMap);
        pullActionToListByInstances(pageListenerInstances, originActionList, dependencies);
        return {
            originComponentList: res.originComponentList.concat(originComponentList),
            originActionList: res.originActionList.concat(originActionList),
        };
    }, {
        originComponentList: [],
        originActionList: [],
    });
    /**
     * 补充 $template 可能用到的组件
     */
    if (usedCmps) {
        const current = new Set(originComponentList.map((item) => item.sourceKey));
        const sourceKey = {};
        for (const libName in usedCmps) {
            if (libName !== '$template') {
                for (let item of Array.from(usedCmps[libName])) {
                    const key = `${libName}:${item}`;
                    if (!current.has(key) && !sourceKey[key]) {
                        sourceKey[key] = true;
                        pullComponentToListByInstance(key, originComponentList, dependencies);
                    }
                }
            }
        }
    }
    const useComponents = [];
    const usedCompMap = originComponentList.reduce((map, compItem) => {
        const sourceKey = `${compItem.materialName}:${compItem.name}`;
        if (!map[sourceKey]) {
            useComponents.push(compItem);
            map[sourceKey] = true;
        }
        return map;
    }, {});
    const originEntryMap = {};
    useComponents.forEach((component) => {
        if (!originEntryMap[component.materialName]) {
            const info = {
                materialName: component.materialName,
                version: component.materialVersion,
                entries: component.entries,
            };
            originEntryMap[component.materialName] = info;
        }
    });
    const useMaterialActions = [];
    originActionList.reduce((map, action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            const sourceKey = `${action.materialName}:${action.name}`;
            if (!map[sourceKey]) {
                useMaterialActions.push(action);
                map[sourceKey] = true;
            }
        }
        return map;
    }, {});
    const officialLib = dependencies.find((lib) => lib.name === config_1.OFFICIAL_LIB_KEY);
    useMaterialActions.forEach((action) => {
        if (!originEntryMap[action.materialName]) {
            const info = {
                materialName: action.materialName,
                version: action.materialVersion,
                entries: action.entries,
            };
            originEntryMap[action.materialName] = info;
        }
    });
    if (officialLib && !originEntryMap[config_1.OFFICIAL_LIB_KEY]) {
        originEntryMap[config_1.OFFICIAL_LIB_KEY] = {
            materialName: officialLib.name,
            version: officialLib.version,
            entries: officialLib.entries,
        };
    }
    const entryImportStringArr = isSandbox ? [] : getEntryImportStringArr(Object.values(originEntryMap));
    const componentImportStringArr = isSandbox
        ? useComponents.map((compItem) => {
            return `const ${(0, util_1.upperFirst)(compItem.variableName)} = window["@weapps-materials-main-${compItem.materialName}"].components["${compItem.name}"]`;
        })
        : getComponentImportStringArr(useComponents);
    const actionImportStringArr = isSandbox
        ? useMaterialActions.map((action) => {
            return `const ${action.variableName} = window["@weapps-materials-main-${action.materialName}"].actions["${action.name}"]`;
        })
        : getActionImportStringArr(useMaterialActions);
    unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr);
    // # all templates to be generated
    const templatesData = {
        /**
         * 使用 render 进行构建的话
         * 外部不再生成 app/global-api.js
         */
        // 'app/global-api.tpl.js': {
        //   appId: appKey,
        //   subPackageName: rootPath,
        //   domain,
        //   appConfig: JsonToStringWithVariableName({
        //     id: appKey,
        //     envId: appData.envId,
        //     clientId: appData.extra?.clientId,
        //     loginConfigVersion: appData.extra?.loginConfigVersion,
        //     staticResourceDomain: domain || '',
        //     envVersion: deployMode === DEPLOY_MODE.UPLOAD ? 'production' : 'preview',
        //     pages: appData.pageInstanceList
        //       .sort((item) => (item.isHome ? -1 : 1))
        //       .map((item) => ({
        //         id: item.id,
        //         type: (item as any).data?.isCustomLoginPage ? 'login' : '',
        //       })),
        //   }),
        // },
        'app/handlers.tpl.js': {
            pageModules,
        },
        'app/common.tpl.js': {
            mods: appData.codeModules
                .filter((m) => m.type === cals_1.ECodeType.NORMAL_MODULE && m.name !== cals_1.ECodeName.PLACEHOLDER)
                .map((m) => m.name),
        },
        'datasources/config.tpl.js': {
            appID: appKey,
            envID: appData.envId,
            isProd: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD,
            clientID: ((_a = appData.extra) === null || _a === void 0 ? void 0 : _a.enableLoginStatusShare) ? appData.envId : (_b = appData.extra) === null || _b === void 0 ? void 0 : _b.clientId,
            isPrivateMode,
        },
        'datasources/datasource-profiles.tpl.js': {
            datasourceProfiles: JSON.stringify((0, dataSource_1.getDatasourceProfiles)(
            /**
             * 精简文件生成，新 cloud-sdk 不依赖 profile 了
             * 2022.08.24 设计态获取数据 outParams.propterties 运行态数据 outParams.items
             * 因此继续使用设计态数据
             * 2023.05.26 新版组件不再依赖此类数据，为了精简，判断不再注入
             */
            // [],
            staticDatasourceProfile ? appData.datasources || [] : []), null, 2),
        },
        'datasources/dataset-profiles.tpl.js': {
            datasetProfiles: (0, util_1.JsonToStringWithVariableName)((0, dataSource_1.getDatasetProfiles)(appData, [appData])),
        },
        'utils/adapter/dependencies.tpl.jsx': {
            upperFirst: util_1.upperFirst,
            OFFICIAL_LIB_KEY: config_1.OFFICIAL_LIB_KEY,
            entryImports: entryImportStringArr.join(';\n'),
            componentImports: componentImportStringArr.join(';\n'),
            actionImports: actionImportStringArr.join(';\n'),
            useComponents,
            useMaterialActions,
            controlMap: JSON.stringify(dependencies.reduce((map, lib) => {
                map[`${lib.name}@${lib.version}`] = {
                    schemaVersion: lib.schemaVersion,
                    components: lib.components
                        .filter((item) => {
                        return usedCompMap[`${lib.name}:${item.name}`];
                    })
                        .map((item) => {
                        const { name, dataForm, properties, emitEvents, events, meta, compConfig, inputProps, syncProps, isContainer, } = item;
                        const { name: metaName, title, platform, shortcut } = meta || {};
                        return {
                            name,
                            dataForm: Object.entries(dataForm || {}).reduce((map, [key, config]) => {
                                const { type, ['x-slot-scope']: xSlotScope, ['x-slot-directives']: xSlotDirectives, syncProp, inputProp, } = config;
                                const filteredDataForm = {
                                    type,
                                    'x-slot-scope': xSlotScope,
                                    'x-slot-directives': xSlotDirectives,
                                    syncProp,
                                    inputProp,
                                };
                                map[key] = filteredDataForm;
                                return map;
                            }, {}),
                            properties: properties
                                ? {
                                    ...properties,
                                    properties: Object.entries(properties.properties || {}).reduce((map, [key, config]) => {
                                        const { type, title } = config;
                                        map[key] = {
                                            type,
                                            title,
                                        };
                                        return map;
                                    }, {}),
                                }
                                : undefined,
                            emitEvents,
                            events,
                            meta: meta
                                ? {
                                    name: metaName,
                                    title,
                                    platform,
                                    shortcut,
                                }
                                : undefined,
                            compConfig,
                            inputProps,
                            syncProps,
                            isContainer,
                        };
                    }),
                };
                return map;
            }, {}), undefined, 2),
        },
    };
    if (!rootPath) {
        templatesData['index.jsx'] = {
            ...yyptConfig,
            i18nConfig,
            isSandbox,
            isBuildApp: buildTypeList.includes(common_1.BuildType.APP),
            subAppDataList,
            adminPortalKey: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD ? appKey : `${appKey}-preview`,
            processCssUnit,
        };
    }
    fileCodeMap = generateFilesFromTemplate({
        templatesData,
        fileCodeMap,
        baseTargetPath: appBuildDir,
    });
}
function getComponentImportStringArr(components, componentImportStringArr = []) {
    components.map(async (component) => {
        const { name, materialName, materialVersion, variableName, entries } = component;
        // const fullName = `${materialName}_${name}`
        // 这里将头字母变成大写是为了能在jsx中以<XXX/>去引用组件
        let importString = '';
        if (entries === null || entries === void 0 ? void 0 : entries.components) {
            importString = `import { ${name} as ${(0, util_1.upperFirst)(variableName)} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.components)}'`;
        }
        else if (entries === null || entries === void 0 ? void 0 : entries.entry) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            importString = `const { ${name}: ${(0, util_1.upperFirst)(variableName)} } = ${componentsLibVariableName}.components`;
        }
        else {
            importString = `import ${(0, util_1.upperFirst)(variableName)} from '/src/libraries/${`${materialName}@${materialVersion}`}/components/${name}'`;
        }
        if (!componentImportStringArr.includes(importString)) {
            componentImportStringArr.push(importString);
        }
    });
    return componentImportStringArr.filter((item) => !!item);
}
function getEntryImportStringArr(materialInfoList = [], entryImportStringArr = []) {
    materialInfoList.forEach(({ materialName, version, entries }) => {
        if (!((entries === null || entries === void 0 ? void 0 : entries.actions) && (entries === null || entries === void 0 ? void 0 : entries.components)) && (entries === null || entries === void 0 ? void 0 : entries.entry)) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            const importComponentLibString = `import ${componentsLibVariableName} from '${path_1.default.posix.join(`/src/libraries/${materialName}@${version}`, entries === null || entries === void 0 ? void 0 : entries.entry)}'`;
            if (!entryImportStringArr.includes(importComponentLibString)) {
                entryImportStringArr.push(importComponentLibString);
            }
        }
    });
    return entryImportStringArr || [];
}
function getActionImportStringArr(originActionList, actionImportStringArr = []) {
    originActionList.map((action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            pushActionToImportStringArr(action, actionImportStringArr);
        }
    });
    return actionImportStringArr;
}
function pushActionToImportStringArr(listenerInstance, actionImportStringArr) {
    const { name, materialName, materialVersion, variableName, entries } = listenerInstance;
    const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
    const importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
        ? `import { ${name} as ${variableName} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
        : (entries === null || entries === void 0 ? void 0 : entries.entry)
            ? `const { ${name}: ${variableName} } = ${componentsLibVariableName}.actions`
            : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.push(importString);
}
function unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr) {
    let importString = '';
    if (officialLib) {
        const { entries } = officialLib;
        const componentsLibVariableName = (0, util_1.camelcase)(`${officialLib.name}`);
        importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
            ? `import * as $$OFFICIAL_ACTION from '${path_1.default.posix.join(`/src/libraries/${officialLib.name}@${officialLib.version}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
            : (entries === null || entries === void 0 ? void 0 : entries.entry)
                ? `const $$OFFICIAL_ACTION  = ${componentsLibVariableName}.actions`
                : '';
        // TODO: 最早的散装函数需要全部手动引用
        // : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    }
    else {
        importString = `const $$OFFICIAL_ACTION = {}`;
    }
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.unshift(importString);
}
function generateFilesFromTemplate({ templatesData, fileCodeMap = {}, baseTargetPath = '/src', }) {
    // eslint-disable-next-line no-restricted-syntax
    for (let file in templatesData) {
        let [templatePath, targetPath] = file.split('|').filter((item) => item === null || item === void 0 ? void 0 : item.trim());
        if (!/^\/src\//.test(templatePath)) {
            templatePath = path_1.default.posix.join('/src', templatePath);
        }
        if (!targetPath) {
            targetPath = templatePath.replace(_1.TEMPLATE_REGEX, '$1');
        }
        if (/^\/src\//.test(targetPath)) {
            targetPath = path_1.default.relative('/src', targetPath);
        }
        if (!template_1.default[templatePath]) {
            throw new Error(`not found ${templatePath}`);
        }
        const tplStr = template_1.default[templatePath].code;
        const generatedCode = (0, lodash_1.template)(tplStr, { interpolate: /<%=([\s\S]+?)%>/g })(templatesData[file]);
        const outFile = path_1.default.join(baseTargetPath, targetPath);
        fileCodeMap[outFile] = {
            code: generatedCode,
        };
    }
    return fileCodeMap;
}
exports.generateFilesFromTemplate = generateFilesFromTemplate;
