"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HACK_FIX_LOWCODE_OUT = exports.HACK_FIX_LOWCODE_IN = exports.getExtByType = exports.originJsonToFileList = void 0;
const cals_1 = require("@cloudbase/cals");
/**
 * WeApps 的文件系统
 * 配置文件
 * LowCode
 * JsonSchema
 */
/**
 * 将原本的 JSON SCHEMA 拆解为文件列表
 */
function originJsonToFileList(appData) {
    const configFiles = handleConfigJson(appData);
    const pageFiles = handlePageJson(appData);
    const lowCodeFiles = handleLowCodeFile(appData);
    return [...configFiles, ...pageFiles, ...lowCodeFiles];
}
exports.originJsonToFileList = originJsonToFileList;
/**
 * TODO: 将文件列表组合成 JSON SCHEMA
 */
// export function fileListToOriginJson(fileList: IWeAppFile[]) {}
// 处理配置类型文件
function handleConfigJson(appData) {
    // 处理全局配置
    const homePageItem = appData.pageInstanceList.find((page) => !!page.isHome);
    const globalConfig = {
        appConfig: appData.appConfig,
        selectedPageId: appData.selectedPageId,
        npmDependencies: appData.npmDependencies,
        homePage: homePageItem === null || homePageItem === void 0 ? void 0 : homePageItem.id,
        themeVars: appData.themeVars,
        presetColors: appData.presetColors,
    };
    const globalFile = {
        path: 'global/app-config',
        name: 'app-config',
        type: 'json',
        code: JSON.stringify(globalConfig, null, 2),
        pageId: 'global',
    };
    // 处理页面配置
    const pagesConfig = appData.pageInstanceList.map((page) => {
        const pageConfig = {
            data: page.data,
            commonStyle: page.commonStyle,
        };
        return {
            path: `${page.id}/page-config`,
            name: 'page-config',
            type: 'json',
            code: JSON.stringify(pageConfig, null, 2),
            pageId: page.id,
        };
    });
    return [globalFile].concat(pagesConfig);
}
// 处理页面类型
function handlePageJson(appData) {
    return appData.pageInstanceList.map((page) => {
        const pageJsonFile = {
            name: 'page',
            pageId: page.id,
            path: `${page.id}/page`,
            type: 'json',
            code: JSON.stringify(page.componentInstances || {}, null, 2),
        };
        return pageJsonFile;
    });
}
// 处理低代码类型
function handleLowCodeFile(appData) {
    const globalLowCodes = appData.lowCodes;
    const pagesLowCodes = appData.pageInstanceList.reduce((pageLowcode, page) => {
        return pageLowcode.concat(page.lowCodes);
    }, []);
    return [...globalLowCodes, ...pagesLowCodes];
}
function initOriginJson(fileList) {
    const data = {
        pageInstanceList: [],
        appConfig: {},
        lowCodes: getCommonLowCodes('global'),
        npmDependencies: {},
        plugins: [],
    };
    // 初始化页面信息
    fileList
        .filter(({ pageId }) => pageId !== 'global')
        .forEach(({ pageId }) => {
        // 将页面类型的结构初始化好
        const pageInstance = data.pageInstanceList.find((page) => page.id === pageId);
        if (!pageInstance) {
            data.pageInstanceList.push({
                id: pageId,
                data: {},
                componentInstances: {},
                lowCodes: getCommonLowCodes(pageId),
                isHome: false,
                listeners: [],
                commonStyle: {},
                pluginInstances: [],
                styleBindPath: '',
            });
        }
    });
    return data;
}
function getCommonLowCodes(pageId) {
    const isGlobal = pageId === 'global';
    const commonLowCodes = [];
    if (isGlobal) {
        commonLowCodes.concat([
            {
                code: 'export default {}',
                name: cals_1.ECodeName.PLACEHOLDER,
                pageId: 'global',
                type: cals_1.ECodeType.NORMAL_MODULE,
                path: `global/common/${cals_1.ECodeName.PLACEHOLDER}`,
                system: false,
            },
        ]);
    }
    else {
        commonLowCodes.concat([
            {
                code: 'export default {}',
                name: cals_1.ECodeName.PLACEHOLDER,
                pageId,
                path: `${pageId}/handler/${cals_1.ECodeName.PLACEHOLDER}`,
                type: cals_1.ECodeType.HANDLER_FN,
                system: false,
            },
        ]);
    }
    return commonLowCodes;
}
// 只处理非 js 文件的
// 'rematch' | 'rematch-action' | 'computed' | 'general-func' | 'general' | 'lifecycle' | 'config' | 'state' | 'handler-fn' | 'normal-module' | 'app-style' | 'style' | 'theme'
function getExtByType(type) {
    switch (type) {
        case 'app-style':
        case cals_1.ECodeType.STYLE:
        case cals_1.ECodeType.THEME:
            return '.less';
        case 'config':
        case 'json':
        case 'app-config':
        case 'page-config':
            return '.json';
        default:
            return '.js';
    }
}
exports.getExtByType = getExtByType;
// HACK: 当前编辑器端文件目录不规范，需要兼容一下目录
// 修复接收低代码
function HACK_FIX_LOWCODE_IN(code = '') {
    return code.replace('../../common/', '../../../common/');
}
exports.HACK_FIX_LOWCODE_IN = HACK_FIX_LOWCODE_IN;
function HACK_FIX_LOWCODE_OUT(code = '') {
    return code.replace('../../../common/', '../../common/');
}
exports.HACK_FIX_LOWCODE_OUT = HACK_FIX_LOWCODE_OUT;
